@@ -225,6 +225,16 @@ func TestParsePub(t *testing.T) {
 	}
 }
 
+// https://www.twistlock.com/labs-blog/finding-dos-vulnerability-nats-go-fuzz-cve-2019-13126/
+func TestParsePubSizeOverflow(t *testing.T) {
+	c := dummyClient()
+
+	pub := []byte("PUB foo 3333333333333333333333333333333333333333333333333333333333333333\r\n")
+	if err := c.parse(pub); err == nil {
+		t.Fatalf("Expected an error")
+	}
+}
+
 func TestParsePubArg(t *testing.T) {
 	c := dummyClient()
 
@@ -235,70 +245,38 @@ func TestParsePubArg(t *testing.T) {
 		size    int
 		szb     string
 	}{
-		{arg: "a 2",
-			subject: "a", reply: "", size: 2, szb: "2"},
-		{arg: "a 222",
-			subject: "a", reply: "", size: 222, szb: "222"},
-		{arg: "foo 22",
-			subject: "foo", reply: "", size: 22, szb: "22"},
-		{arg: " foo 22",
-			subject: "foo", reply: "", size: 22, szb: "22"},
-		{arg: "foo 22 ",
-			subject: "foo", reply: "", size: 22, szb: "22"},
-		{arg: "foo   22",
-			subject: "foo", reply: "", size: 22, szb: "22"},
-		{arg: " foo 22 ",
-			subject: "foo", reply: "", size: 22, szb: "22"},
-		{arg: " foo   22 ",
-			subject: "foo", reply: "", size: 22, szb: "22"},
-		{arg: "foo bar 22",
-			subject: "foo", reply: "bar", size: 22, szb: "22"},
-		{arg: " foo bar 22",
-			subject: "foo", reply: "bar", size: 22, szb: "22"},
-		{arg: "foo bar 22 ",
-			subject: "foo", reply: "bar", size: 22, szb: "22"},
-		{arg: "foo  bar  22",
-			subject: "foo", reply: "bar", size: 22, szb: "22"},
-		{arg: " foo bar 22 ",
-			subject: "foo", reply: "bar", size: 22, szb: "22"},
-		{arg: "  foo   bar  22  ",
-			subject: "foo", reply: "bar", size: 22, szb: "22"},
-		{arg: "  foo   bar  2222  ",
-			subject: "foo", reply: "bar", size: 2222, szb: "2222"},
-		{arg: "  foo     2222  ",
-			subject: "foo", reply: "", size: 2222, szb: "2222"},
-		{arg: "a\t2",
-			subject: "a", reply: "", size: 2, szb: "2"},
-		{arg: "a\t222",
-			subject: "a", reply: "", size: 222, szb: "222"},
-		{arg: "foo\t22",
-			subject: "foo", reply: "", size: 22, szb: "22"},
-		{arg: "\tfoo\t22",
-			subject: "foo", reply: "", size: 22, szb: "22"},
-		{arg: "foo\t22\t",
-			subject: "foo", reply: "", size: 22, szb: "22"},
-		{arg: "foo\t\t\t22",
-			subject: "foo", reply: "", size: 22, szb: "22"},
-		{arg: "\tfoo\t22\t",
-			subject: "foo", reply: "", size: 22, szb: "22"},
-		{arg: "\tfoo\t\t\t22\t",
-			subject: "foo", reply: "", size: 22, szb: "22"},
-		{arg: "foo\tbar\t22",
-			subject: "foo", reply: "bar", size: 22, szb: "22"},
-		{arg: "\tfoo\tbar\t22",
-			subject: "foo", reply: "bar", size: 22, szb: "22"},
-		{arg: "foo\tbar\t22\t",
-			subject: "foo", reply: "bar", size: 22, szb: "22"},
-		{arg: "foo\t\tbar\t\t22",
-			subject: "foo", reply: "bar", size: 22, szb: "22"},
-		{arg: "\tfoo\tbar\t22\t",
-			subject: "foo", reply: "bar", size: 22, szb: "22"},
-		{arg: "\t \tfoo\t \t \tbar\t \t22\t \t",
-			subject: "foo", reply: "bar", size: 22, szb: "22"},
-		{arg: "\t\tfoo\t\t\tbar\t\t2222\t\t",
-			subject: "foo", reply: "bar", size: 2222, szb: "2222"},
-		{arg: "\t \tfoo\t \t \t\t\t2222\t \t",
-			subject: "foo", reply: "", size: 2222, szb: "2222"},
+		{arg: "a 2", subject: "a", reply: "", size: 2, szb: "2"},
+		{arg: "a 222", subject: "a", reply: "", size: 222, szb: "222"},
+		{arg: "foo 22", subject: "foo", reply: "", size: 22, szb: "22"},
+		{arg: " foo 22", subject: "foo", reply: "", size: 22, szb: "22"},
+		{arg: "foo 22 ", subject: "foo", reply: "", size: 22, szb: "22"},
+		{arg: "foo   22", subject: "foo", reply: "", size: 22, szb: "22"},
+		{arg: " foo 22 ", subject: "foo", reply: "", size: 22, szb: "22"},
+		{arg: " foo   22 ", subject: "foo", reply: "", size: 22, szb: "22"},
+		{arg: "foo bar 22", subject: "foo", reply: "bar", size: 22, szb: "22"},
+		{arg: " foo bar 22", subject: "foo", reply: "bar", size: 22, szb: "22"},
+		{arg: "foo bar 22 ", subject: "foo", reply: "bar", size: 22, szb: "22"},
+		{arg: "foo  bar  22", subject: "foo", reply: "bar", size: 22, szb: "22"},
+		{arg: " foo bar 22 ", subject: "foo", reply: "bar", size: 22, szb: "22"},
+		{arg: "  foo   bar  22  ", subject: "foo", reply: "bar", size: 22, szb: "22"},
+		{arg: "  foo   bar  2222  ", subject: "foo", reply: "bar", size: 2222, szb: "2222"},
+		{arg: "  foo     2222  ", subject: "foo", reply: "", size: 2222, szb: "2222"},
+		{arg: "a\t2", subject: "a", reply: "", size: 2, szb: "2"},
+		{arg: "a\t222", subject: "a", reply: "", size: 222, szb: "222"},
+		{arg: "foo\t22", subject: "foo", reply: "", size: 22, szb: "22"},
+		{arg: "\tfoo\t22", subject: "foo", reply: "", size: 22, szb: "22"},
+		{arg: "foo\t22\t", subject: "foo", reply: "", size: 22, szb: "22"},
+		{arg: "foo\t\t\t22", subject: "foo", reply: "", size: 22, szb: "22"},
+		{arg: "\tfoo\t22\t", subject: "foo", reply: "", size: 22, szb: "22"},
+		{arg: "\tfoo\t\t\t22\t", subject: "foo", reply: "", size: 22, szb: "22"},
+		{arg: "foo\tbar\t22", subject: "foo", reply: "bar", size: 22, szb: "22"},
+		{arg: "\tfoo\tbar\t22", subject: "foo", reply: "bar", size: 22, szb: "22"},
+		{arg: "foo\tbar\t22\t", subject: "foo", reply: "bar", size: 22, szb: "22"},
+		{arg: "foo\t\tbar\t\t22", subject: "foo", reply: "bar", size: 22, szb: "22"},
+		{arg: "\tfoo\tbar\t22\t", subject: "foo", reply: "bar", size: 22, szb: "22"},
+		{arg: "\t \tfoo\t \t \tbar\t \t22\t \t", subject: "foo", reply: "bar", size: 22, szb: "22"},
+		{arg: "\t\tfoo\t\t\tbar\t\t2222\t\t", subject: "foo", reply: "bar", size: 2222, szb: "2222"},
+		{arg: "\t \tfoo\t \t \t\t\t2222\t \t", subject: "foo", reply: "", size: 2222, szb: "2222"},
 	} {
 		t.Run(test.arg, func(t *testing.T) {
 			if err := c.processPub([]byte(test.arg)); err != nil {
@@ -329,8 +307,266 @@ func TestParsePubBadSize(t *testing.T) {
 	}
 }
 
+func TestParseHeaderPub(t *testing.T) {
+	c := dummyClient()
+	c.headers = true
+
+	hpub := []byte("HPUB foo 12 17\r\nname:derek\r\nHELLO\r")
+	if err := c.parse(hpub); err != nil || c.state != MSG_END_N {
+		t.Fatalf("Unexpected: %d : %v\n", c.state, err)
+	}
+	if !bytes.Equal(c.pa.subject, []byte("foo")) {
+		t.Fatalf("Did not parse subject correctly: 'foo' vs '%s'", c.pa.subject)
+	}
+	if c.pa.reply != nil {
+		t.Fatalf("Did not parse reply correctly: 'nil' vs '%s'", c.pa.reply)
+	}
+	if c.pa.hdr != 12 {
+		t.Fatalf("Did not parse msg header size correctly: 12 vs %d", c.pa.hdr)
+	}
+	if !bytes.Equal(c.pa.hdb, []byte("12")) {
+		t.Fatalf("Did not parse or capture the header size as bytes correctly: %q", c.pa.hdb)
+	}
+	if c.pa.size != 17 {
+		t.Fatalf("Did not parse msg size correctly: 17 vs %d", c.pa.size)
+	}
+
+	// Clear snapshots
+	c.argBuf, c.msgBuf, c.state = nil, nil, OP_START
+
+	hpub = []byte("HPUB foo INBOX.22 12 17\r\nname:derek\r\nHELLO\r")
+	if err := c.parse(hpub); err != nil || c.state != MSG_END_N {
+		t.Fatalf("Unexpected: %d : %v\n", c.state, err)
+	}
+	if !bytes.Equal(c.pa.subject, []byte("foo")) {
+		t.Fatalf("Did not parse subject correctly: 'foo' vs '%s'", c.pa.subject)
+	}
+	if !bytes.Equal(c.pa.reply, []byte("INBOX.22")) {
+		t.Fatalf("Did not parse reply correctly: 'INBOX.22' vs '%s'", c.pa.reply)
+	}
+	if c.pa.hdr != 12 {
+		t.Fatalf("Did not parse msg header size correctly: 12 vs %d", c.pa.hdr)
+	}
+	if !bytes.Equal(c.pa.hdb, []byte("12")) {
+		t.Fatalf("Did not parse or capture the header size as bytes correctly: %q", c.pa.hdb)
+	}
+	if c.pa.size != 17 {
+		t.Fatalf("Did not parse msg size correctly: 17 vs %d", c.pa.size)
+	}
+
+	// Clear snapshots
+	c.argBuf, c.msgBuf, c.state = nil, nil, OP_START
+
+	hpub = []byte("HPUB foo INBOX.22 0 5\r\nHELLO\r")
+	if err := c.parse(hpub); err != nil || c.state != MSG_END_N {
+		t.Fatalf("Unexpected: %d : %v\n", c.state, err)
+	}
+	if !bytes.Equal(c.pa.subject, []byte("foo")) {
+		t.Fatalf("Did not parse subject correctly: 'foo' vs '%s'\n", c.pa.subject)
+	}
+	if !bytes.Equal(c.pa.reply, []byte("INBOX.22")) {
+		t.Fatalf("Did not parse reply correctly: 'INBOX.22' vs '%s'\n", c.pa.reply)
+	}
+	if c.pa.hdr != 0 {
+		t.Fatalf("Did not parse msg header size correctly: 0 vs %d\n", c.pa.hdr)
+	}
+	if !bytes.Equal(c.pa.hdb, []byte("0")) {
+		t.Fatalf("Did not parse or capture the header size as bytes correctly: %q", c.pa.hdb)
+	}
+	if c.pa.size != 5 {
+		t.Fatalf("Did not parse msg size correctly: 5 vs %d\n", c.pa.size)
+	}
+}
+
+func TestParseHeaderPubArg(t *testing.T) {
+	c := dummyClient()
+	c.headers = true
+
+	for _, test := range []struct {
+		arg     string
+		subject string
+		reply   string
+		hdr     int
+		size    int
+		szb     string
+	}{
+		{arg: "a 2 4", subject: "a", reply: "", hdr: 2, size: 4, szb: "4"},
+		{arg: "a 22 222", subject: "a", reply: "", hdr: 22, size: 222, szb: "222"},
+		{arg: "foo 3 22", subject: "foo", reply: "", hdr: 3, size: 22, szb: "22"},
+		{arg: " foo   1 22", subject: "foo", reply: "", hdr: 1, size: 22, szb: "22"},
+		{arg: "foo 0   22 ", subject: "foo", reply: "", hdr: 0, size: 22, szb: "22"},
+		{arg: "foo  0     22", subject: "foo", reply: "", hdr: 0, size: 22, szb: "22"},
+		{arg: " foo 1 22 ", subject: "foo", reply: "", hdr: 1, size: 22, szb: "22"},
+		{arg: " foo   3 22 ", subject: "foo", reply: "", hdr: 3, size: 22, szb: "22"},
+		{arg: "foo bar 1 22", subject: "foo", reply: "bar", hdr: 1, size: 22, szb: "22"},
+		{arg: " foo bar 11 22", subject: "foo", reply: "bar", hdr: 11, size: 22, szb: "22"},
+		{arg: "foo bar 11 22 ", subject: "foo", reply: "bar", hdr: 11, size: 22, szb: "22"},
+		{arg: "foo  bar  11  22", subject: "foo", reply: "bar", hdr: 11, size: 22, szb: "22"},
+		{arg: " foo bar  11 22 ", subject: "foo", reply: "bar", hdr: 11, size: 22, szb: "22"},
+		{arg: "  foo   bar  11   22  ", subject: "foo", reply: "bar", hdr: 11, size: 22, szb: "22"},
+		{arg: "  foo   bar  22   2222  ", subject: "foo", reply: "bar", hdr: 22, size: 2222, szb: "2222"},
+		{arg: "  foo 1    2222  ", subject: "foo", reply: "", hdr: 1, size: 2222, szb: "2222"},
+		{arg: "a\t2\t22", subject: "a", reply: "", hdr: 2, size: 22, szb: "22"},
+		{arg: "a\t2\t\t222", subject: "a", reply: "", hdr: 2, size: 222, szb: "222"},
+		{arg: "foo\t2 22", subject: "foo", reply: "", hdr: 2, size: 22, szb: "22"},
+		{arg: "\tfoo\t11\t  22", subject: "foo", reply: "", hdr: 11, size: 22, szb: "22"},
+		{arg: "foo\t11\t22\t", subject: "foo", reply: "", hdr: 11, size: 22, szb: "22"},
+		{arg: "foo\t\t\t11 22", subject: "foo", reply: "", hdr: 11, size: 22, szb: "22"},
+		{arg: "\tfoo\t11\t \t 22\t", subject: "foo", reply: "", hdr: 11, size: 22, szb: "22"},
+		{arg: "\tfoo\t\t\t11 22\t", subject: "foo", reply: "", hdr: 11, size: 22, szb: "22"},
+		{arg: "foo\tbar\t2 22", subject: "foo", reply: "bar", hdr: 2, size: 22, szb: "22"},
+		{arg: "\tfoo\tbar\t11\t22", subject: "foo", reply: "bar", hdr: 11, size: 22, szb: "22"},
+		{arg: "foo\tbar\t11\t\t22\t ", subject: "foo", reply: "bar", hdr: 11, size: 22, szb: "22"},
+		{arg: "foo\t\tbar\t\t11\t\t\t22", subject: "foo", reply: "bar", hdr: 11, size: 22, szb: "22"},
+		{arg: "\tfoo\tbar\t11\t22\t", subject: "foo", reply: "bar", hdr: 11, size: 22, szb: "22"},
+		{arg: "\t \tfoo\t \t \tbar\t \t11\t 22\t \t", subject: "foo", reply: "bar", hdr: 11, size: 22, szb: "22"},
+		{arg: "\t\tfoo\t\t\tbar\t\t22\t\t\t2222\t\t", subject: "foo", reply: "bar", hdr: 22, size: 2222, szb: "2222"},
+		{arg: "\t \tfoo\t \t \t\t\t11\t\t 2222\t \t", subject: "foo", reply: "", hdr: 11, size: 2222, szb: "2222"},
+	} {
+		t.Run(test.arg, func(t *testing.T) {
+			if err := c.processHeaderPub([]byte(test.arg)); err != nil {
+				t.Fatalf("Unexpected parse error: %v\n", err)
+			}
+			if !bytes.Equal(c.pa.subject, []byte(test.subject)) {
+				t.Fatalf("Mismatched subject: '%s'\n", c.pa.subject)
+			}
+			if !bytes.Equal(c.pa.reply, []byte(test.reply)) {
+				t.Fatalf("Mismatched reply subject: '%s'\n", c.pa.reply)
+			}
+			if !bytes.Equal(c.pa.szb, []byte(test.szb)) {
+				t.Fatalf("Bad size buf: '%s'\n", c.pa.szb)
+			}
+			if c.pa.hdr != test.hdr {
+				t.Fatalf("Bad header size: %d\n", c.pa.hdr)
+			}
+			if c.pa.size != test.size {
+				t.Fatalf("Bad size: %d\n", c.pa.size)
+			}
+		})
+	}
+}
+
+func TestParseRoutedHeaderMsg(t *testing.T) {
+	c := dummyRouteClient()
+	c.route = &route{}
+
+	pub := []byte("HMSG $foo foo 10 8\r\nXXXhello\r")
+	if err := c.parse(pub); err == nil {
+		t.Fatalf("Expected an error")
+	}
+
+	// Clear snapshots
+	c.argBuf, c.msgBuf, c.state = nil, nil, OP_START
+
+	pub = []byte("HMSG $foo foo 3 8\r\nXXXhello\r")
+	err := c.parse(pub)
+	if err != nil || c.state != MSG_END_N {
+		t.Fatalf("Unexpected: %d : %v\n", c.state, err)
+	}
+	if !bytes.Equal(c.pa.account, []byte("$foo")) {
+		t.Fatalf("Did not parse account correctly: '$foo' vs '%s'\n", c.pa.account)
+	}
+	if !bytes.Equal(c.pa.subject, []byte("foo")) {
+		t.Fatalf("Did not parse subject correctly: 'foo' vs '%s'\n", c.pa.subject)
+	}
+	if c.pa.reply != nil {
+		t.Fatalf("Did not parse reply correctly: 'nil' vs '%s'\n", c.pa.reply)
+	}
+	if c.pa.hdr != 3 {
+		t.Fatalf("Did not parse header size correctly: 3 vs %d\n", c.pa.hdr)
+	}
+	if c.pa.size != 8 {
+		t.Fatalf("Did not parse msg size correctly: 8 vs %d\n", c.pa.size)
+	}
+
+	// Clear snapshots
+	c.argBuf, c.msgBuf, c.state = nil, nil, OP_START
+
+	pub = []byte("HMSG $G foo.bar INBOX.22 3 14\r\nOK:hello world\r")
+	err = c.parse(pub)
+	if err != nil || c.state != MSG_END_N {
+		t.Fatalf("Unexpected: %d : %v\n", c.state, err)
+	}
+	if !bytes.Equal(c.pa.account, []byte("$G")) {
+		t.Fatalf("Did not parse account correctly: '$G' vs '%s'\n", c.pa.account)
+	}
+	if !bytes.Equal(c.pa.subject, []byte("foo.bar")) {
+		t.Fatalf("Did not parse subject correctly: 'foo' vs '%s'\n", c.pa.subject)
+	}
+	if !bytes.Equal(c.pa.reply, []byte("INBOX.22")) {
+		t.Fatalf("Did not parse reply correctly: 'INBOX.22' vs '%s'\n", c.pa.reply)
+	}
+	if c.pa.hdr != 3 {
+		t.Fatalf("Did not parse header size correctly: 3 vs %d\n", c.pa.hdr)
+	}
+	if c.pa.size != 14 {
+		t.Fatalf("Did not parse msg size correctly: 14 vs %d\n", c.pa.size)
+	}
+
+	// Clear snapshots
+	c.argBuf, c.msgBuf, c.state = nil, nil, OP_START
+
+	pub = []byte("HMSG $G foo.bar + reply baz 3 14\r\nOK:hello world\r")
+	err = c.parse(pub)
+	if err != nil || c.state != MSG_END_N {
+		t.Fatalf("Unexpected: %d : %v\n", c.state, err)
+	}
+	if !bytes.Equal(c.pa.account, []byte("$G")) {
+		t.Fatalf("Did not parse account correctly: '$G' vs '%s'\n", c.pa.account)
+	}
+	if !bytes.Equal(c.pa.subject, []byte("foo.bar")) {
+		t.Fatalf("Did not parse subject correctly: 'foo' vs '%s'\n", c.pa.subject)
+	}
+	if !bytes.Equal(c.pa.reply, []byte("reply")) {
+		t.Fatalf("Did not parse reply correctly: 'reply' vs '%s'\n", c.pa.reply)
+	}
+	if len(c.pa.queues) != 1 {
+		t.Fatalf("Expected 1 queue, got %d", len(c.pa.queues))
+	}
+	if !bytes.Equal(c.pa.queues[0], []byte("baz")) {
+		t.Fatalf("Did not parse queues correctly: 'baz' vs '%q'\n", c.pa.queues[0])
+	}
+	if c.pa.hdr != 3 {
+		t.Fatalf("Did not parse header size correctly: 3 vs %d\n", c.pa.hdr)
+	}
+	if c.pa.size != 14 {
+		t.Fatalf("Did not parse msg size correctly: 14 vs %d\n", c.pa.size)
+	}
+
+	// Clear snapshots
+	c.argBuf, c.msgBuf, c.state = nil, nil, OP_START
+
+	pub = []byte("HMSG $G foo.bar | baz 3 14\r\nOK:hello world\r")
+	err = c.parse(pub)
+	if err != nil || c.state != MSG_END_N {
+		t.Fatalf("Unexpected: %d : %v\n", c.state, err)
+	}
+	if !bytes.Equal(c.pa.account, []byte("$G")) {
+		t.Fatalf("Did not parse account correctly: '$G' vs '%s'\n", c.pa.account)
+	}
+	if !bytes.Equal(c.pa.subject, []byte("foo.bar")) {
+		t.Fatalf("Did not parse subject correctly: 'foo' vs '%s'\n", c.pa.subject)
+	}
+	if !bytes.Equal(c.pa.reply, []byte("")) {
+		t.Fatalf("Did not parse reply correctly: '' vs '%s'\n", c.pa.reply)
+	}
+	if len(c.pa.queues) != 1 {
+		t.Fatalf("Expected 1 queue, got %d", len(c.pa.queues))
+	}
+	if !bytes.Equal(c.pa.queues[0], []byte("baz")) {
+		t.Fatalf("Did not parse queues correctly: 'baz' vs '%q'\n", c.pa.queues[0])
+	}
+	if c.pa.hdr != 3 {
+		t.Fatalf("Did not parse header size correctly: 3 vs %d\n", c.pa.hdr)
+	}
+	if c.pa.size != 14 {
+		t.Fatalf("Did not parse msg size correctly: 14 vs %d\n", c.pa.size)
+	}
+}
+
 func TestParseRouteMsg(t *testing.T) {
 	c := dummyRouteClient()
+	c.route = &route{}
 
 	pub := []byte("MSG $foo foo 5\r\nhello\r")
 	err := c.parse(pub)
@@ -541,7 +777,7 @@ func TestProtoSnippet(t *testing.T) {
 	}
 
 	for _, tt := range tests {
-		got := protoSnippet(tt.input, sample)
+		got := protoSnippet(tt.input, PROTO_SNIPPET_SIZE, sample)
 		if tt.expected != got {
 			t.Errorf("Expected protocol snippet to be %s when start=%d but got %s\n", tt.expected, tt.input, got)
 		}
@@ -577,12 +813,63 @@ func TestParseOK(t *testing.T) {
 }
 
 func TestMaxControlLine(t *testing.T) {
-	c := dummyClient()
-	c.mcl = 8
+	for _, test := range []struct {
+		name       string
+		kind       int
+		shouldFail bool
+	}{
+		{"client", CLIENT, true},
+		{"leaf", LEAF, false},
+		{"route", ROUTER, false},
+		{"gateway", GATEWAY, false},
+	} {
+		t.Run(test.name, func(t *testing.T) {
+			pub := []byte("PUB foo.bar.baz 2\r\nok\r\n")
 
-	pub := []byte("PUB foo.bar 11\r")
-	err := c.parse(pub)
-	if !ErrorIs(err, ErrMaxControlLine) {
-		t.Fatalf("Expected an error parsing longer than expected control line")
+			setupClient := func() *client {
+				c := dummyClient()
+				c.setNoReconnect()
+				c.flags.set(connectReceived)
+				c.kind = test.kind
+				switch test.kind {
+				case ROUTER:
+					c.route = &route{}
+				case GATEWAY:
+					c.gw = &gateway{outbound: false, connected: true, insim: make(map[string]*insie)}
+				}
+				c.mcl = 8
+				return c
+			}
+
+			c := setupClient()
+			// First try with a partial:
+			// PUB foo.bar.baz 2\r\nok\r\n
+			// .............^
+			err := c.parse(pub[:14])
+			switch test.shouldFail {
+			case true:
+				if !ErrorIs(err, ErrMaxControlLine) {
+					t.Fatalf("Expected an error parsing longer than expected control line")
+				}
+			case false:
+				if err != nil {
+					t.Fatalf("Should not have failed, got %v", err)
+				}
+			}
+
+			// Now with full protocol (no split) and we should still enforce.
+			c = setupClient()
+			err = c.parse(pub)
+			switch test.shouldFail {
+			case true:
+				if !ErrorIs(err, ErrMaxControlLine) {
+					t.Fatalf("Expected an error parsing longer than expected control line")
+				}
+			case false:
+				if err != nil {
+					t.Fatalf("Should not have failed, got %v", err)
+				}
+			}
+		})
 	}
 }
