@@ -28,6 +28,7 @@ import (
 type stateMachine interface {
 	server() *Server
 	node() RaftNode
+	waitGroup() *sync.WaitGroup
 	// This will call forward as needed so can be called on any node.
 	propose(data []byte)
 	// When entries have been committed and can be applied.
@@ -157,9 +158,13 @@ func (c *cluster) createRaftGroupWithPeers(name string, servers []*Server, smf s
 // Driver program for the state machine.
 // Should be run in its own go routine.
 func smLoop(sm stateMachine) {
-	s, n := sm.server(), sm.node()
+	s, n, wg := sm.server(), sm.node(), sm.waitGroup()
 	qch, lch, aq := n.QuitC(), n.LeadChangeC(), n.ApplyQ()
 
+	// Wait group used to allow waiting until we exit from here.
+	wg.Add(1)
+	defer wg.Done()
+
 	for {
 		select {
 		case <-s.quitCh:
@@ -185,6 +190,7 @@ type stateAdder struct {
 	sync.Mutex
 	s   *Server
 	n   RaftNode
+	wg  sync.WaitGroup
 	cfg *RaftConfig
 	sum int64
 	lch chan bool
@@ -196,12 +202,19 @@ func (a *stateAdder) server() *Server {
 	defer a.Unlock()
 	return a.s
 }
+
 func (a *stateAdder) node() RaftNode {
 	a.Lock()
 	defer a.Unlock()
 	return a.n
 }
 
+func (a *stateAdder) waitGroup() *sync.WaitGroup {
+	a.Lock()
+	defer a.Unlock()
+	return &a.wg
+}
+
 func (a *stateAdder) propose(data []byte) {
 	a.Lock()
 	defer a.Unlock()
@@ -243,9 +256,10 @@ func (a *stateAdder) proposeDelta(delta int64) {
 
 // Stop the group.
 func (a *stateAdder) stop() {
-	a.Lock()
-	defer a.Unlock()
-	a.n.Stop()
+	n, wg := a.node(), a.waitGroup()
+	n.Stop()
+	n.WaitForStop()
+	wg.Wait()
 }
 
 // Restart the group
@@ -273,6 +287,11 @@ func (a *stateAdder) restart() {
 		panic(err)
 	}
 
+	// Must reset in-memory state.
+	// A real restart would not preserve it, but more importantly we have no way to detect if we
+	// already applied an entry. So, the sum must only be updated based on append entries or snapshots.
+	a.sum = 0
+
 	a.n, err = a.s.startRaftNode(globalAccountName, a.cfg, pprofLabels{})
 	if err != nil {
 		panic(err)
@@ -317,3 +336,40 @@ func (rg smGroup) waitOnTotal(t *testing.T, expected int64) {
 func newStateAdder(s *Server, cfg *RaftConfig, n RaftNode) stateMachine {
 	return &stateAdder{s: s, n: n, cfg: cfg, lch: make(chan bool, 1)}
 }
+
+func initSingleMemRaftNode(t *testing.T) (*raft, func()) {
+	t.Helper()
+	n, c := initSingleMemRaftNodeWithCluster(t)
+	cleanup := func() {
+		c.shutdown()
+	}
+	return n, cleanup
+}
+
+func initSingleMemRaftNodeWithCluster(t *testing.T) (*raft, *cluster) {
+	t.Helper()
+	c := createJetStreamClusterExplicit(t, "R3S", 3)
+	s := c.servers[0] // RunBasicJetStreamServer not available
+
+	ms, err := newMemStore(&StreamConfig{Name: "TEST", Storage: MemoryStorage})
+	require_NoError(t, err)
+	cfg := &RaftConfig{Name: "TEST", Store: t.TempDir(), Log: ms}
+
+	err = s.bootstrapRaftNode(cfg, nil, false)
+	require_NoError(t, err)
+	n, err := s.initRaftNode(globalAccountName, cfg, pprofLabels{})
+	require_NoError(t, err)
+
+	return n, c
+}
+
+// Encode an AppendEntry.
+// An AppendEntry is encoded into a buffer and that's stored into the WAL.
+// This is a helper function to generate that buffer.
+func encode(t *testing.T, ae *appendEntry) *appendEntry {
+	t.Helper()
+	buf, err := ae.encode(nil)
+	require_NoError(t, err)
+	ae.buf = buf
+	return ae
+}
